@@ -1,16 +1,20 @@
+# encoding: utf-8
+
 """
 Module to handle command line argument parsing, for all front-ends.
 """
 
-from __future__ import with_statement, print_function
+from __future__ import print_function, absolute_import
+
+import code
+import imp
 import os
 import sys
-import code
 from optparse import OptionParser, OptionGroup
 
-from bpython import __version__
-from bpython.config import default_config_path, loadini, Struct
-from bpython.translations import _
+from . import __version__
+from .config import default_config_path, loadini, Struct
+from .translations import _
 
 
 class OptionParserFailed(ValueError):
@@ -88,9 +92,8 @@ def parse(args, extras=None, ignore_stdin=False):
         os.execv(sys.executable, [sys.executable] + args)
 
     if options.version:
-        print('bpython version', __version__, end=" ")
-        print('on top of Python', sys.version.split()[0])
-        print('(C) 2008-2015 Bob Farrell, Andreas Stuehrk et al. '
+        print(version_banner())
+        print('(C) 2008-2016 Bob Farrell, Andreas Stuehrk, Sebastian Ramacher, Thomas Ballinger, et al. '
               'See AUTHORS for detail.')
         raise SystemExit
 
@@ -114,6 +117,9 @@ def exec_code(interpreter, args):
         source = sourcefile.read()
     old_argv, sys.argv = sys.argv, args
     sys.path.insert(0, os.path.abspath(os.path.dirname(args[0])))
+    mod = imp.new_module('__console__')
+    sys.modules['__console__'] = mod
+    interpreter.locals = mod.__dict__
     interpreter.locals['__file__'] = args[0]
     interpreter.runsource(source, args[0], 'exec')
     sys.argv = old_argv
