@@ -5,7 +5,7 @@ import logging
 import itertools
 from six.moves import range
 
-from curtsies import fsarray, fmtstr
+from curtsies import fsarray, fmtstr, FSArray
 from curtsies.formatstring import linesplit
 from curtsies.fmtfuncs import bold
 
@@ -56,7 +56,7 @@ def paginate(rows, matches, current, words_wide):
     return matches[per_page * current_page:per_page * (current_page + 1)]
 
 
-def matches_lines(rows, columns, matches, current, config, format):
+def matches_lines(rows, columns, matches, current, config, match_format):
     highlight_color = func_for_letter(config.color_scheme['operator'].lower())
 
     if not matches:
@@ -64,22 +64,22 @@ def matches_lines(rows, columns, matches, current, config, format):
     color = func_for_letter(config.color_scheme['main'])
     max_match_width = max(len(m) for m in matches)
     words_wide = max(1, (columns - 1) // (max_match_width + 1))
-    matches = [format(m) for m in matches]
+    matches = [match_format(m) for m in matches]
     if current:
-        current = format(current)
+        current = match_format(current)
 
     matches = paginate(rows, matches, current, words_wide)
 
-    matches_lines = [fmtstr(' ').join(color(m.ljust(max_match_width))
-                                      if m != current
-                                      else highlight_color(
-                                          m.ljust(max_match_width))
-                                      for m in matches[i:i+words_wide])
+    result = [fmtstr(' ').join(color(m.ljust(max_match_width))
+                                     if m != current
+                                     else highlight_color(
+                                         m.ljust(max_match_width))
+                                     for m in matches[i:i + words_wide])
                      for i in range(0, len(matches), words_wide)]
 
     logger.debug('match: %r' % current)
-    logger.debug('matches_lines: %r' % matches_lines)
-    return matches_lines
+    logger.debug('matches_lines: %r' % result)
+    return result
 
 
 def formatted_argspec(funcprops, arg_pos, columns, config):
@@ -161,6 +161,13 @@ def formatted_argspec(funcprops, arg_pos, columns, config):
 
 
 def formatted_docstring(docstring, columns, config):
+    if isinstance(docstring, bytes):
+        docstring = docstring.decode('utf8')
+    elif isinstance(docstring, str if py3 else unicode):
+        pass
+    else:
+        # TODO: fail properly here and catch possible exceptions in callers.
+        return []
     color = func_for_letter(config.color_scheme['comment'])
     return sum(([color(x) for x in (display_linize(line, columns) if line else
                                     fmtstr(''))]
@@ -168,17 +175,17 @@ def formatted_docstring(docstring, columns, config):
 
 
 def paint_infobox(rows, columns, matches, funcprops, arg_pos, match, docstring,
-                  config, format):
+                  config, match_format):
     """Returns painted completions, funcprops, match, docstring etc."""
     if not (rows and columns):
-        return fsarray(0, 0)
+        return FSArray(0, 0)
     width = columns - 4
     from_argspec = (formatted_argspec(funcprops, arg_pos, width, config)
                     if funcprops else [])
     from_doc = (formatted_docstring(docstring, width, config)
                 if docstring else [])
     from_matches = (matches_lines(max(1, rows - len(from_argspec) - 2),
-                                  width, matches, match, config, format)
+                                  width, matches, match, config, match_format)
                     if matches else [])
 
     lines = from_argspec + from_matches + from_doc
@@ -211,7 +218,7 @@ def paint_last_events(rows, columns, names, config):
         return fsarray([])
     width = min(max(len(name) for name in names), columns - 2)
     output_lines = []
-    output_lines.append(config.left_top_corner+config.top_border * width +
+    output_lines.append(config.left_top_corner + config.top_border * width +
                         config.right_top_corner)
     for name in reversed(names[max(0, len(names) - (rows - 2)):]):
         output_lines.append(config.left_border + name[:width].center(width) +
