@@ -23,13 +23,14 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+from __future__ import absolute_import
+
 import code
 import inspect
 import io
 import os
 import pkgutil
 import pydoc
-import re
 import shlex
 import subprocess
 import sys
@@ -39,23 +40,27 @@ import time
 import traceback
 from itertools import takewhile
 from six import itervalues
+from types import ModuleType
 
 from pygments.token import Token
 
-from bpython import autocomplete
-from bpython import inspection
-from bpython._py3compat import PythonLexer, py3, prepare_for_exec
-from bpython.clipboard import get_clipboard, CopyFailed
-from bpython.config import getpreferredencoding
-from bpython.formatter import Parenthesis
-from bpython.history import History
-from bpython.paste import PasteHelper, PastePinnwand, PasteFailed
-from bpython.patch_linecache import filename_for_console_input
-from bpython.translations import _, ngettext
-from bpython import simpleeval
+from . import autocomplete
+from . import inspection
+from ._py3compat import PythonLexer, py3, prepare_for_exec
+from .clipboard import get_clipboard, CopyFailed
+from .config import getpreferredencoding
+from .formatter import Parenthesis
+from .history import History
+from .lazyre import LazyReCompile
+from .paste import PasteHelper, PastePinnwand, PasteFailed
+from .patch_linecache import filename_for_console_input
+from .translations import _, ngettext
+from . import simpleeval
 
 
 class RuntimeTimer(object):
+    """Calculate running time"""
+
     def __init__(self):
         self.reset_timer()
         self.time = time.monotonic if hasattr(time, 'monotonic') else time.time
@@ -77,18 +82,40 @@ class RuntimeTimer(object):
 
 
 class Interpreter(code.InteractiveInterpreter):
+    """Source code interpreter for use in bpython."""
+
+    bpython_input_re = LazyReCompile(r'<bpython-input-\d+>')
 
     def __init__(self, locals=None, encoding=None):
-        """The syntaxerror callback can be set at any time and will be called
+        """Constructor.
+
+        The optional 'locals' argument specifies the dictionary in which code
+        will be executed; it defaults to a newly created dictionary with key
+        "__name__" set to "__main__".
+
+        The syntaxerror callback can be set at any time and will be called
         on a caught syntax error. The purpose for this in bpython is so that
         the repl can be instantiated after the interpreter (which it
         necessarily must be with the current factoring) and then an exception
-        callback can be added to the Interpeter instance afterwards - more
+        callback can be added to the Interpreter instance afterwards - more
         specifically, this is so that autoindentation does not occur after a
-        traceback."""
+        traceback.
 
-        self.encoding = encoding or sys.getdefaultencoding()
+        encoding is only used in Python 2, where it may be necessary to add an
+        encoding comment to a source bytestring before running it.
+        encoding must be a bytestring in Python 2 because it will be templated
+        into a bytestring source as part of an encoding comment.
+        """
+
+        self.encoding = encoding or getpreferredencoding()
         self.syntaxerror_callback = None
+
+        if locals is None:
+            # instead of messing with sys.modules, we should modify sys.modules
+            # in the interpreter instance
+            sys.modules['__main__'] = main_mod = ModuleType('__main__')
+            locals = main_mod.__dict__
+
         # Unfortunately code.InteractiveInterpreter is a classic class, so no
         # super()
         code.InteractiveInterpreter.__init__(self, locals)
@@ -98,15 +125,56 @@ class Interpreter(code.InteractiveInterpreter):
         self.running_time = 0
 
     def runsource(self, source, filename=None, symbol='single',
-                  encode=True):
+                  encode='auto'):
         """Execute Python code.
 
         source, filename and symbol are passed on to
-        code.InteractiveInterpreter.runsource. If encode is True, the source
-        will be encoded. On Python 3.X, encode will be ignored."""
-        if not py3 and encode:
-            source = u'# coding: %s\n\n%s' % (self.encoding, source)
-            source = source.encode(self.encoding)
+        code.InteractiveInterpreter.runsource. If encode is True,
+        an encoding comment will be added to the source.
+        On Python 3.X, encode will be ignored.
+
+        encode should only be used for interactive interpreter input,
+        files should always already have an encoding comment or be ASCII.
+        By default an encoding line will be added if no filename is given.
+
+        In Python 3, source must be a unicode string
+        In Python 2, source may be latin-1 bytestring or unicode string,
+        following the interface of code.InteractiveInterpreter.
+
+        Because adding an encoding comment to a unicode string in Python 2
+        would cause a syntax error to be thrown which would reference code
+        the user did not write, setting encoding to True when source is a
+        unicode string in Python 2 will throw a ValueError."""
+        # str means bytestring in Py2
+        if encode and not py3 and isinstance(source, unicode):
+            if encode != 'auto':
+                raise ValueError("can't add encoding line to unicode input")
+            encode = False
+        if encode and filename is not None:
+            # files have encoding comments or implicit encoding of ASCII
+            if encode != 'auto':
+                raise ValueError(
+                    "shouldn't add encoding line to file contents")
+            encode = False
+
+        if encode and not py3 and isinstance(source, str):
+            # encoding makes sense for bytestrings, so long as there
+            # isn't already an encoding comment
+            comment = inspection.get_encoding_comment(source)
+            if comment:
+                # keep the existing encoding comment, but add two lines
+                # because this interp always adds 2 to stack trace line
+                # numbers in Python 2
+                source = source.replace(comment, b'%s\n\n' % comment, 1)
+            else:
+                source = b'# coding: %s\n\n%s' % (self.encoding, source)
+        elif not py3 and filename is None:
+            # 2 blank lines still need to be added
+            # because this interpreter always adds 2 to stack trace line
+            # numbers in Python 2 when the filename is "<input>"
+            newlines = u'\n\n' if isinstance(source, unicode) else b'\n\n'
+            source = newlines + source
+            # we know we're in Python 2 here, so ok to reference unicode
         if filename is None:
             filename = filename_for_console_input(source)
         with self.timer:
@@ -120,10 +188,10 @@ class Interpreter(code.InteractiveInterpreter):
         if self.syntaxerror_callback is not None:
             self.syntaxerror_callback()
 
-        type, value, sys.last_traceback = sys.exc_info()
-        sys.last_type = type
+        exc_type, value, sys.last_traceback = sys.exc_info()
+        sys.last_type = exc_type
         sys.last_value = value
-        if filename and type is SyntaxError:
+        if filename and exc_type is SyntaxError:
             # Work hard to stuff the correct filename in the exception
             try:
                 msg, (dummy_filename, lineno, offset, line) = value.args
@@ -132,15 +200,15 @@ class Interpreter(code.InteractiveInterpreter):
                 pass
             else:
                 # Stuff in the right filename and right lineno
-                if not py3:
-                    lineno -= 2
                 # strip linecache line number
-                if re.match(r'<bpython-input-\d+>', filename):
+                if self.bpython_input_re.match(filename):
                     filename = '<input>'
+                if filename == '<input>' and not py3:
+                    lineno -= 2
                 value = SyntaxError(msg, (filename, lineno, offset, line))
                 sys.last_value = value
-        list = traceback.format_exception_only(type, value)
-        self.writetb(list)
+        exc_formatted = traceback.format_exception_only(exc_type, value)
+        self.writetb(exc_formatted)
 
     def showtraceback(self):
         """This needs to override the default traceback thing
@@ -156,13 +224,12 @@ class Interpreter(code.InteractiveInterpreter):
 
             for i, (fname, lineno, module, something) in enumerate(tblist):
                 # strip linecache line number
-                if re.match(r'<bpython-input-\d+>', fname):
+                if self.bpython_input_re.match(fname):
                     fname = '<input>'
                     tblist[i] = (fname, lineno, module, something)
                 # Set the right lineno (encoding header adds an extra line)
-                if not py3:
-                    if fname == '<input>':
-                        tblist[i] = (fname, lineno - 2, module, something)
+                if fname == '<input>' and not py3:
+                    tblist[i] = (fname, lineno - 2, module, something)
 
             l = traceback.format_list(tblist)
             if l:
@@ -436,7 +503,7 @@ class Repl(object):
             with io.open(filename, 'rt', encoding=encoding) as f:
                 source = f.read()
                 if not py3:
-                    # Python 2.6 and early 2.7.X need bytes.
+                    # Early Python 2.7.X need bytes.
                     source = source.encode(encoding)
                 self.interp.runsource(source, filename, 'exec', encode=False)
 
@@ -556,7 +623,7 @@ class Repl(object):
                 try:
                     fake_cursor = self.current_line.index(func) + len(func)
                     f = simpleeval.evaluate_current_attribute(
-                            fake_cursor, self.current_line, self.interp.locals)
+                        fake_cursor, self.current_line, self.interp.locals)
                 except simpleeval.EvaluationError:
                     return False
         except Exception:
@@ -725,13 +792,13 @@ class Repl(object):
             indentation = 0
         return indentation
 
-    def formatforfile(self, s):
+    def formatforfile(self, session_ouput):
         """Format the stdout buffer to something suitable for writing to disk,
         i.e. without >>> and ... at input lines and with "# OUT: " prepended to
         output lines."""
 
         def process():
-            for line in s.split('\n'):
+            for line in session_ouput.split('\n'):
                 if line.startswith(self.ps1):
                     yield line[len(self.ps1):]
                 elif line.startswith(self.ps2):
@@ -772,11 +839,11 @@ class Repl(object):
                 self.interact.notify(_('Save cancelled.'))
                 return
 
-        s = self.formatforfile(self.getstdout())
+        stdout_text = self.formatforfile(self.getstdout())
 
         try:
             with open(fn, mode) as f:
-                f.write(s)
+                f.write(stdout_text)
         except IOError as e:
             self.interact.notify(_("Error writing file '%s': %s") % (fn, e))
         else:
@@ -814,8 +881,8 @@ class Repl(object):
         if s == self.prev_pastebin_content:
             self.interact.notify(_('Duplicate pastebin. Previous URL: %s. '
                                    'Removal URL: %s') %
-                                  (self.prev_pastebin_url,
-                                   self.prev_removal_url), 10)
+                                 (self.prev_pastebin_url,
+                                  self.prev_removal_url), 10)
             return self.prev_pastebin_url
 
         self.interact.notify(_('Posting data to pastebin...'))
@@ -854,8 +921,9 @@ class Repl(object):
         return more
 
     def insert_into_history(self, s):
+        pythonhist = os.path.expanduser(self.config.hist_file)
         try:
-            self.rl_history.append_reload_and_write(s, self.config.hist_file,
+            self.rl_history.append_reload_and_write(s, pythonhist,
                                                     getpreferredencoding())
         except RuntimeError as e:
             self.interact.notify(u"%s" % (e, ))
@@ -1023,10 +1091,10 @@ class Repl(object):
 
     def clear_current_line(self):
         """This is used as the exception callback for the Interpreter instance.
-        It prevents autoindentation from occuring after a traceback."""
+        It prevents autoindentation from occurring after a traceback."""
 
-    def send_to_external_editor(self, text, filename=None):
-        """Returns modified text from an editor, or the oriignal text if editor
+    def send_to_external_editor(self, text):
+        """Returns modified text from an editor, or the original text if editor
         exited with non-zero"""
 
         encoding = getpreferredencoding()
@@ -1054,7 +1122,7 @@ class Repl(object):
         return subprocess.call(args) == 0
 
     def edit_config(self):
-        if not (os.path.isfile(self.config.config_path)):
+        if not os.path.isfile(self.config.config_path):
             if self.interact.confirm(_("Config file does not exist - create "
                                        "new from default? (y/N)")):
                 try:
@@ -1062,7 +1130,6 @@ class Repl(object):
                                                       'sample-config')
                     if py3:  # py3 files need unicode
                         default_config = default_config.decode('ascii')
-                    bpython_dir, script_name = os.path.split(__file__)
                     containing_dir = os.path.dirname(
                         os.path.abspath(self.config.config_path))
                     if not os.path.exists(containing_dir):
@@ -1096,10 +1163,10 @@ def next_indentation(line, tab_length):
     return indentation
 
 
-def next_token_inside_string(s, inside_string):
+def next_token_inside_string(code_string, inside_string):
     """Given a code string s and an initial state inside_string, return
     whether the next token will be inside a string or not."""
-    for token, value in PythonLexer().get_tokens(s):
+    for token, value in PythonLexer().get_tokens(code_string):
         if token is Token.String:
             value = value.lstrip('bBrRuU')
             if value in ['"""', "'''", '"', "'"]:
